@@ -104,6 +104,8 @@ sub make_pcaps {
 ############################ work as server
 sub serve {
     my ($addr,$sslargs) = @_;
+    my %iscat = map { $_->ID => 1 } App::DubiousHTTP::Tests->categories;
+
     App::DubiousHTTP::TestServer->run($addr, $sslargs, sub {
 	my ($path,$listen,$rqhdr,$payload,$ssl) = @_;
 	return "HTTP/1.0 404 not found\r\n\r\n" if $path eq '/favicon.ico';
@@ -116,7 +118,6 @@ sub serve {
 	    my $body = '';
 	    print STDERR $rqhdr;
 	    if ($what ne 'part') {
-		print STDERR $rqhdr;
 		$body = "<!doctype html>"
 		    ."<h1>Thanks for providing us with the feedback.</h1>";
 	    }
@@ -127,7 +128,7 @@ sub serve {
 
 	local $BASE_URL = "http://$listen";
 	my ($auto,$src,$cat,$page,$spec,$qstring) = $path =~m{\A / 
-	    (?:(auto)/|((?:raw)?src)/)?
+	    (?:auto(js|img|html|xhr|)/|((?:raw)?src)/)?
 	    ([^/]+)
 	    (?: / ([^/\?]*))?
 	    (?: / ([^\?]*))?
@@ -137,8 +138,10 @@ sub serve {
 	    $_ = '' if ! defined $_;
 	}
 
-	return App::DubiousHTTP::Tests->auto($cat,$page,$spec,$qstring,$rqhdr)
-	    if $auto;
+	if (defined $auto && ($iscat{$cat} || $cat eq 'all')) {
+	    return App::DubiousHTTP::Tests->auto(
+		$auto || 'xhr',$cat,$page,$spec,$qstring,$rqhdr)
+	}
 
 	if ( $page eq 'ALL' && $cat ) {
 	    for ( App::DubiousHTTP::Tests->categories ) {
