@@ -37,7 +37,7 @@ More information about bypassing firewalls using interpretation differences can
 be found <a href="http://noxxi.de/research/semantic-gap.html">here</a>.
 </p>
 
-<h2>Bulk test with virus payload</h2>
+<h2>Bulk test with virus payload (XHR)</h2>
 
 <p>
 This bulk test tries to transfer the <a
@@ -61,7 +61,7 @@ the evasion.
 </p>
 <p id=test_virus class=runtest><a href="/auto/all/eicar.txt">Run Test with <strong>EICAR test virus</strong> payload</a></p>
 
-<h2>Bulk test with innocent payload</h2>
+<h2>Bulk test with innocent payload (XHR)</h2>
 
 <p>
 This is the same bulk test as the previous one but this time the payload is
@@ -73,6 +73,24 @@ uncommon or invalid HTTP response.
 </p>
 <p id=test_novirus class=runtest><a href="/auto/all/novirus.txt">Run Test with <strong>innocent</strong> payload</a></p>
 
+<h2>Bulk test with innocent Javascript</h2>
+
+<p>
+Contrary to the previous bulk tests this one is not done with XMLHttpRequest but
+instead it analyzes which responses will successfully be interpreted as
+JavaScript by the browser.
+</p>
+<p id=test_js class=runtest><a href="/autojs/all/set_success.js">Run Test with
+innocent JavaScript payload</a></p>
+
+<h2>Bulk test with innocent Image</h2>
+
+<p>
+This bulk test will use img-Tags to download an innocent image to check which
+uncommen responses can be used to load images.
+</p>
+<p id=test_js class=runtest><a href="/autoimg/all/ok.png">Run Test with
+innocent image payload</a></p>
 
 <h2>Non-Bulk tests</h2>
 
@@ -115,12 +133,27 @@ HTML
 }
 
 sub auto {
+    my $self = shift;
+    my $type = shift;
+    return $self->auto_xhr(@_) if $type eq 'xhr';
+    return $self->auto_js(@_) if $type eq 'js';
+    return $self->auto_img(@_) if $type eq 'img';
+    die;
+}
+
+sub auto_xhr {
     my ($self,$cat,$page,$spec,$qstring,$rqhdr) = @_;
     $page ||= 'eicar.txt';
     my $html = _auto_static_html();
     my ($hdr,$body,$isbad) = content($page);
     $html .= "<script>\n";
 
+    if (my ($vendor,$msg) = vendor_notice($CLIENTIP)) {
+	warn "VENDOR_NOTICE($CLIENTIP) $vendor\n";
+	$msg =~s{(\\|")|(\r)|(\n)|(\t)}{ "\\".($1||($2?'r':$3?'n':'t'))}eg;
+	$html .= "vendor_notice(\"$msg\");\n";
+    }
+
     my ($accept) = $rqhdr =~m{^Accept:[ \t]*([^\r\n]+)}mi;
     if ($qstring =~m{(?:^|\&)accept=([^&]*)}) {
 	($accept = $1) =~s{(?:%([a-f\d]{2})|(\+))}{ $2 ? ' ' : chr(hex($1)) }esg;
@@ -165,11 +198,139 @@ sub auto {
 	$html;
 }
 
+sub auto_img {
+    my ($self,$cat) = @_;
+    _auto_imgjs($cat, 'ok.png', sub {
+	my ($test,$page,$rand) = @_;
+	my $url = $test->url($page);
+	my $xid = quotemeta(html_escape($test->LONG_ID));
+	return "<img src='$url?rand=$rand' onload='set_success(\"$xid\");' onerror='set_fail(\"$xid\");' />\n";
+    });
+}
+
+sub auto_js {
+    my ($self,$cat) = @_;
+    _auto_imgjs($cat, 'set_success.js', sub {
+	my ($test,$page,$rand) = @_;
+	my $url = $test->url($page);
+	my $xid = quotemeta(html_escape($test->LONG_ID));
+	return "<script src='$url?rand=$rand' onerror='set_fail(\"$xid\");' onload='set_done(\"$xid\");' onreadystatechange='set_done(\"$xid\");'></script>\n";
+    });
+}
+
+sub _auto_imgjs {
+    my ($cat,$page,$mkhtml) = @_;
+
+    my $jsglob = <<'JS';
+
+var fast_feedback = 0;
+var tests = [];
+var done = 0;
+var check_timer;
+var timer = 0;
+var maxwait = 20;
+
+function set_success(xid) { check_done(xid, 'success') }
+function set_fail(xid)    { check_done(xid, 'fail') }
+function set_done(xid)    { check_done(xid) }
+
+check_timer = function() {
+    timer++;
+    if (timer<maxwait) {
+	// start timer again
+	window.setTimeout(check_timer,1000);
+	div_process.innerHTML = "Progress: " + (100*done/tests.length).toFixed(1) + "% - " + "waiting(" + (maxwait-timer) + ")";
+	maxwait = 10+(tests.length-done);
+	return;
+    } else if (timer == maxwait) {
+	check_done();
+    }
+};
+
+function check_done(xid,val) {
+    if (xid) {
+	timer = 0;
+	var desc;
+	for(var i=0;i<tests.length;i++) {
+	    if (tests[i]['xid'] == xid) {
+		desc = tests[i]['desc'];
+		add_debug( val + ": " + desc, tests[i]);
+		if (!tests[i]['status']) {
+		    tests[i]['status'] = val || 'unknown';
+		    done++;
+		} else if (val) {
+		    tests[i]['status'] = val;
+		}
+		if (val) {
+		    check_status_noevil(tests[i]);
+		}
+		break;
+	    }
+	}
+
+	if (fast_feedback && results.length > fast_feedback) {
+	    submit_part();
+	}
+	if (done < tests.length) {
+	    div_process.innerHTML = "Progress: " + (100*done/tests.length).toFixed(1) + "% - " + desc;
+	    return;
+	}
+    }
+
+
+    // DONE: remove working node
+    var w = document.getElementById('work');
+    w.parentNode.removeChild(w);
+    div_process.style.display = 'none';
+    add_debug("*done*");
+    timer = maxwait + 100;
+
+    if (done < tests.length) {
+	// fill in the rest
+	for(var i=0;i<tests.length;i++) {
+	    if (!tests[i]['status']) {
+		tests[i]['status'] = 'timeout';
+		add_debug( tests[i]['status'] + ": " + tests[i]['desc'], tests[i]);
+		check_status_noevil(tests[i]);
+	    }
+	}
+    }
+
+    submit_result('/submit_results/' + reference,results);
+}
+
+check_timer();
+JS
+
+    $jsglob .= "fast_feedback = 16384;\n" if $FAST_FEEDBACK;
+    my $html_tests = '';
+    my $rand = rand();
+    for(@cat) {
+	next if $cat ne 'all' && $_->ID ne $cat;
+	for($_->TESTS) {
+	    $jsglob .= 'tests.push({ page: "' . $_->url($page)
+		. '", xid: "'.quotemeta(html_escape($_->LONG_ID))
+		. '", desc: "'.quotemeta(html_escape($_->DESCRIPTION))
+		. '", valid: '.$_->VALID ."});\n";
+	    $html_tests .= $mkhtml->($_,$page,$rand);
+	}
+    }
+
+    my $html = _auto_static_html()."<script>$jsglob</script>\n<div id=work style='display:none;'>$html_tests</div>";
+    return "HTTP/1.0 200 ok\r\n".
+	"Content-type: text/html\r\n".
+	"Content-length: ".length($html)."\r\n".
+	"\r\n".
+	$html;
+}
+
+
 sub _auto_static_html { return <<'HTML'; }
 <!doctype html>
 <meta charset="utf-8">
 <style>
 body      { font-family: Verdana, sans-serif; }
+#vendor_notice  { padding: 2em; margin: 1em; background: #000000; color: #ff0000; font-size: 150%; display: none; }
 #nobad    { padding: 2em; margin: 1em; background: #ff3333; display: none; }
 #nobad div   { font-size: 150%; margin: 0.5em;  }
 #noevade  { padding: 1em; margin: 1em; background: green; display: none; }
@@ -189,6 +350,7 @@ body      { font-family: Verdana, sans-serif; }
 <div id=noscript>
 You need to have JavaScript enabled to run this tests.
 </div>
+<div id=vendor_notice> </div>
 <div id=nobad> </div>
 <div id=urlblock> </div>
 <div id=evasions></td>
@@ -203,6 +365,13 @@ You need to have JavaScript enabled to run this tests.
 
 var time = Date.now || function() { return +new Date; };
 
+function vendor_notice(msg) {
+    var div = document.getElementById('vendor_notice');
+    if (!div) return;
+    div.innerHTML = msg;
+    div.style.display = 'block';
+}
+
 function base64_encode(input,urlsafe) {
     var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
     var output = "";
@@ -293,8 +462,8 @@ function add_notice(m,test) {
 function add_debug(m,test) {
     div_debug.innerHTML = div_debug.innerHTML + m + (test ?
 	"&nbsp;<a class=trylink target=_blank download='" + test['file'] + "' href=" + test['page'] + ">try</a>" +
-	"&nbsp;<a class=srclink target=_blank href=/src" + test['page'] + ">src</a>" +
-	"<br>": "");
+	"&nbsp;<a class=srclink target=_blank href=/src" + test['page'] + ">src</a>"
+	: "" ) + "<br>";
 }
 
 function escapeAttribute(attr) {
@@ -481,11 +650,19 @@ function check_page(req,test,status) {
 	    // add answer to results, maybe we can get the type of firewall from the error message
 	    results = results + "T | " + test['page'] + " | " + result64 + "\n";
 	}
-	return status;
-    }
 
-    // check for standard conformance
-    if (status == 'match') {
+    } else {
+	// check for standard conformance
+	check_status_noevil(test,status);
+    }
+    return status;
+}
+
+function check_status_noevil(test,status) {
+    if (!status) {
+	status = test['status'];
+    }
+    if (status == 'success' || status == 'match') {
 	if (test['valid'] == 0) {
 	    add_warning("success for bad response",test);
 	    results = results + "W | " + status + " | " + test['page'] + " | " + test['desc'] + " | success for bad response\n";
@@ -512,7 +689,6 @@ function check_page(req,test,status) {
 	    results = results + "I | " + status + " | " + test['page'] + " | " + test['desc'] + " | ok\n";
 	}
     }
-    return status;
 }
 
 var rand = Math.random();
@@ -653,5 +829,60 @@ document.getElementById('noscript').style.display = 'none';
 </script>
 HTML
 
+{
+
+    my (%msg,@map);
+    sub vendor_notice {
+	my $srcip = shift or return;
+	$srcip =~m{:} and return; # IPv6 not handled yet here
+	@map || return;
+	my $ipn = 0;
+	$ipn = 256*$ipn + $_ for split(m{\.+},$srcip);
+	for(@map) {
+	    next if $ipn<$_->[1];
+	    next if $ipn>$_->[2];
+	    my $vendor = $_->[0];
+	    return ($vendor,$msg{$vendor});
+	}
+	return;
+    }
+
+    # load notice on startup
+    if (open(my $fh,'<','vendor_notice.txt')) {
+	my $vendor;
+	while (<$fh>) {
+	    if ($vendor) {
+		if (m{^=end\s*$}) {
+		    $vendor = undef;
+		} else {
+		    $msg{$vendor} .= $_;
+		}
+	    } elsif ( m{^=begin (\S+)}) {
+		$vendor = $1;
+		die "message for vendor $vendor already loaded"
+		    if $msg{$vendor};
+	    } elsif (my ($ip0,$net,$ip1,$vendor) =
+		m{^=map\s+([\d\.]+)(?:/(\d+)|\s*-\s*([\d\.]+))\s+(\S+)}) {
+		for my $ip ($ip0,$ip1) {
+		    defined $ip or next;
+		    my @ip = split(m{\.+},$ip);
+		    push @ip,0 while @ip<4;
+		    $ip = 0;
+		    $ip = 256*$ip + $_ for @ip;
+		}
+		$ip1 = $ip0 + (2 << (32-$net)) if defined $net;
+		push @map,[ $vendor,$ip0,$ip1 ];
+	    } elsif (do { s{#.*}{}; m{\S}}) {
+		die "invalid line $_";
+	    }
+	}
+	for my $vendor (keys %msg) {
+	    die "no source-ip for $vendor defined"
+		if !grep { $_->[0] eq $vendor } @map;
+	}
+	warn "DEBUG: vendor notice loaded for $_\n" for (sort keys %msg);
+    }
+}
+
 
 1;
