@@ -175,7 +175,7 @@ DESC
 sub make_response {
     my ($self,$page,$spec) = @_;
     return make_index_page() if $page eq '';
-    my ($cthdr,$data) = content($page,$spec) or die "unknown page $page";
+    my ($cthdr,$data) = content($page,$self->ID."-".$spec) or die "unknown page $page";
     my $version = '1.1';
     my $te = 'clen';
     my $only = 0;
